@@ -9,10 +9,10 @@ use XML::Parser;
 require Exporter;
 
 our @ISA         = qw(Exporter);
-our %EXPORT_TAGS = ( all => [ qw() ] );
+our %EXPORT_TAGS = ( all => [ qw(slurp_xml) ] );
 our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT      = qw();
-our $VERSION     = '0.25';
+our $VERSION     = '0.26';
 
 sub newhd {
     my $class = shift;
@@ -502,6 +502,42 @@ sub DESTROY {
     }
 }
 
+sub slurp_xml {
+    my ($data, $root, $twigs) = @_;
+
+    # in @$twigs --> remove all spaces and then all leading and trailing '/', then put back a single leading '/'
+    for my $tw (@$twigs) {
+        $tw =~ s{\s}''xmsg;
+        $tw =~ s{\A /+}''xms;
+        $tw =~ s{/+ \z}''xms;
+        $tw = '/'.$tw;
+    }
+
+    my $list   = [];
+    my $branch = [];
+
+    my $rdr = XML::Reader->newhd($data, {filter => 4, using => $root}) or die "Error: $!";
+
+    while ($rdr->iterate) {
+        if ($rdr->path eq '/' and $rdr->is_start) {
+            $branch = [];
+        }
+        if ($rdr->is_value) {
+            for my $i (0..$#$twigs) {
+                if ($rdr->path eq $twigs->[$i]) {
+                    $branch->[$i] .= $rdr->value;
+                }
+            }
+        }
+        if ($rdr->path eq '/' and $rdr->is_end) {
+            push @$list, $branch;
+            $branch = []; # ...only to be on the safe side: empty $branch after it has been pushed on @$list
+        }
+    }
+
+    return $list;
+}
+
 # The package used here - XML::Reader::Token 
 # has been inspired by    XML::TokeParser::Token
 
@@ -1376,6 +1412,65 @@ text-values (see scalar '$count'), so that we can distinguish between the first
       }
   }
 
+=head1 FUNCTIONS
+
+=head2 Function slurp_xml
+
+The function slurp_xml reads an XML file and slurps it into an array-ref. Here is an
+example where we want to slurp the name, the street and the city of all customers in
+the path '/data/order/database/customer':
+
+  use XML::Reader qw(slurp_xml);
+
+  my $line2 = q{
+  <data>
+    <order>
+      <database>
+        <customer name="smith" id="652">
+          <street>high street</street>
+          <city>boston</city>
+        </customer>
+        <customer name="jones" id="184">
+          <street>maple street</street>
+          <city>new york</city>
+        </customer>
+        <customer name="stewart" id="520">
+          <street>ring road</street>
+          <city>dallas</city>
+        </customer>
+      </database>
+    </order>
+    <dummy value="ttt">test</dummy>
+    <supplier>hhh</supplier>
+    <supplier>iii</supplier>
+    <supplier>jjj</supplier>
+  </data>
+  };
+
+  my $aref = slurp_xml(\$line2, '/data/order/database/customer',
+    ['/@name', '/street', '/city']);
+
+  for (@$aref) {
+      printf "Name = %-7s Street = %-12s City = %s\n", $_->[0], $_->[1], $_->[2];
+  }
+
+The first parameter to slurp_xml is the filename (or scalar reference, or open filehandle) of the XML
+that will be slurped. In this case we read from a scalar ref \$line2. The second parameter is the
+root of the sub-tree that we want to slurp (in our case that's '/data/order/database/customer').
+Finally we supply a list of the elements that we want to slurp, relative to the sub-tree. In this
+case it is ['/@name', '/street', '/city'].
+
+Here is the output:
+
+  Name = smith   Street = high street  City = boston
+  Name = jones   Street = maple street City = new york
+  Name = stewart Street = ring road    City = dallas
+
+slurp_xml works similar to L<XML::Simple>, in that it reads all required information in one go into
+an in-memory data structure. The difference, however, is that slurp_xml lets you be specific in what
+you actually want before you do the slurping, so that your in-memory data structure is smaller
+and less complicated.
+
 =head1 AUTHOR
 
 Klaus Eichner, March 2009
@@ -1397,6 +1492,7 @@ DATA_INDENT=>2, which allows for proper indentation in your XML-Output file)
 =head1 SEE ALSO
 
 L<XML::TokeParser>,
+L<XML::Simple>,
 L<XML::Parser>,
 L<XML::Parser::Expat>,
 L<XML::TiePYX>,
